//
// Copyright (C) 2012 Intel Corporation. All Rights Reserved.
//
// The source code contained or described herein and all documents related
// to the source code ("Material") are owned by Intel Corporation or its
// suppliers or licensors.  Title to the Material remains with Intel
// Corporation or its suppliers and licensors.  The Material is protected
// by worldwide copyright laws and treaty provisions.  No part of the
// Material may be used, copied, reproduced, modified, published, uploaded,
// posted, transmitted, distributed, or disclosed in any way without
// Intel's prior express written permission.
//
// No license under any patent, copyright, trade secret or other intellectual
// property right is granted to or conferred upon you by disclosure or
// delivery of the Materials, either expressly, by implication, inducement,
// estoppel or otherwise. Any license under such intellectual property rights
// must be express and approved by Intel in writing.
//
// Unless otherwise agreed by Intel in writing, you may not remove or alter
// this notice or any other notice embedded in Materials by Intel or Intel's
// suppliers or licensors in any way.
//

#ifndef SDLT_ALWAYS_FALSE_VALUE_H
#define SDLT_ALWAYS_FALSE_VALUE_H

#include <type_traits>

#include "../common.h"

namespace sdlt
{
namespace __SDLT_IMPL
{
namespace internal
{

__SDLT_INHERIT_IMPL_BEGIN

template<typename T>
struct always_false_type : public std::false_type
{
};


template<typename T>
struct always_true_type : public std::true_type
{
};

__SDLT_INHERIT_IMPL_END

} // namepace internal
} // namepace __SDLT_IMPL
} // namespace sdlt

#endif // SDLT_ALWAYS_FALSE_VALUE_H
